!Models-3/CMAQ code adapted for MONARCH

!***********************************************************************
!   Portions of Models-3/CMAQ software were developed or based on      *
!   information from various groups: Federal Government employees,     *
!   contractors working on a United States Government contract, and    *
!   non-Federal sources (including research institutions).  These      *
!   research institutions have given the Government permission to      *
!   use, prepare derivative works, and distribute copies of their      *
!   work in Models-3/CMAQ to the public and to permit others to do     *
!   so.  EPA therefore grants similar permissions for use of the       *
!   Models-3/CMAQ software, but users are requested to provide copies  *
!   of derivative works to the Government without restrictions as to   *
!   use by others.  Users are responsible for acquiring their own      *
!   copies of commercial software associated with Models-3/CMAQ and    *
!   for complying with vendor requirements.  Software copyrights by    *
!   the MCNC Environmental Modeling Center are used with their         *
!   permissions subject to the above restrictions.                     *
!***********************************************************************

! RCS file, release, date & time of last delta, author, state, [and locker]
! $Header: /project/work/rep/CCTM/src/chem/ebi_cb05cl_ae5/hrprodloss.F,v 1.2 2008/04/09 20:21:35 sjr Exp $ 

! what(1) key, module and SID; SCCS file; date and time of last delta:
! %W% %P% %G% %U%
!MONARCH
      SUBROUTINE EXT_HRPRODLOSS
!MONARCH
!***********************************************************************
!
!  FUNCTION: To compute the production and loss rates for EBI species
!
!  PRECONDITIONS: For the CB05CL family of mechanisms
!
!  RETURN VALUES: None
!
!  KEY SUBROUTINES/FUNCTIONS CALLED: None
!
!  REVISION HISTORY: Created by EBI solver program, April 1, 2008
!
!***********************************************************************
!MONARCH
      USE EXT_HRDATA
!MONARCH
      IMPLICIT NONE


!.....INCLUDES: NONE


!.....ARGUMENTS: NONE


!.....PARAMETERS: NONE


!.....EXTERNAL FUNCTIONS: NONE


!.....SAVED VARIABLES: NONE
      CHARACTER( 16 ), SAVE  ::  PNAME = 'HRPRODLOSS'   ! Program name


!.....LOCAL VARIABLES:

      INTEGER ESP

!***********************************************************************


      PROD( HNO3    ) =   2.000 * RXRAT(  19 )      ! N2O5=2*HNO3
     &                 +  2.000 * RXRAT(  20 )      ! N2O5=2*HNO3
     &                 +          RXRAT(  28 )      ! NO2+OH=HNO3
     &                 +          RXRAT(  48 )      ! NO3+HO2=HNO3
     &                 +          RXRAT(  61 )      ! NTR+OH=HNO3+HO2+0.33*FORM+...
     &                 +          RXRAT(  77 )      ! FORM+NO3=HNO3+HO2+CO
     &                 +          RXRAT(  85 )      ! ALD2+NO3=HNO3+C2O3
     &                 +          RXRAT( 100 )      ! ALDX+NO3=HNO3+CXO3
     &                 +          RXRAT( 132 )      ! CRES+NO3=HNO3+CRO
     &                 +  0.150 * RXRAT( 147 )      ! NO3+ISPD=0.15*HNO3+...
      LOSS( HNO3    ) =           RXRAT(  29 )      ! HNO3+OH=NO3
     &                 +          RXRAT(  52 )      ! HNO3=OH+NO2

      PROD( H2O2    ) =           RXRAT(  34 )      ! HO2+HO2=H2O2
     &                 +          RXRAT(  35 )      ! HO2+HO2=H2O2
     &                 +          RXRAT(  42 )      ! OH+OH=H2O2
      LOSS( H2O2    ) =           RXRAT(  36 )      ! H2O2=2*OH
     &                 +          RXRAT(  37 )      ! H2O2+OH=HO2
     &                 +          RXRAT(  45 )      ! H2O2+O=OH+HO2

      PROD( XO2     ) =           RXRAT(  63 )      ! ROOH+OH=XO2+0.5*ALD2+0.5*ALDX
     &                 +  0.300 * RXRAT(  70 )      ! MEPX+OH=0.3*XO2+0.7*MEO2+...
     &                 +          RXRAT( 102 )      ! CXO3+NO=XO2+NO2+HO2+ALD2
     &                 +  0.900 * RXRAT( 108 )      ! CXO3+MEO2=0.9*XO2+0.9*ALD2+...
     &                 +  2.000 * RXRAT( 110 )      ! CXO3+CXO3=2*XO2+2*ALD2+2*HO2
     &                 +          RXRAT( 111 )      ! CXO3+C2O3=XO2+MEO2+HO2+ALD2
     &                 +  0.870 * RXRAT( 112 )      ! PAR+OH=0.87*XO2+0.13*XO2N+...
     &                 +  0.960 * RXRAT( 113 )      ! ROR=0.96*XO2+0.6*ALD2+...
     &                 +  0.200 * RXRAT( 116 )      ! O+OLE=0.2*XO2+0.3*ALDX+...
     &                 +  0.800 * RXRAT( 117 )      ! OH+OLE=0.8*XO2+0.33*ALD2+...
     &                 +  0.220 * RXRAT( 118 )      ! O3+OLE=0.22*XO2+0.74*FORM+...
     &                 +  0.910 * RXRAT( 119 )      ! NO3+OLE=0.91*XO2+FORM+NO2+...
     &                 +  0.700 * RXRAT( 120 )      ! O+ETH=0.7*XO2+1.7*HO2+CO+...
     &                 +          RXRAT( 121 )      ! OH+ETH=XO2+1.56*FORM+...
     &                 +          RXRAT( 123 )      ! NO3+ETH=XO2+NO2+2*FORM
     &                 +  0.100 * RXRAT( 124 )      ! IOLE+O=0.1*XO2+0.66*ALDX+...
     &                 +          RXRAT( 125 )      ! IOLE+OH=XO2+0.7*ALDX+HO2+...
     &                 +  0.080 * RXRAT( 128 )      ! TOL+OH=0.08*XO2+0.44*HO2+...
     &                 +  0.600 * RXRAT( 131 )      ! OH+CRES=0.6*XO2+0.4*CRO+...
     &                 +          RXRAT( 136 )      ! OPEN+OH=XO2+2*CO+2*HO2+...
     &                 +  0.030 * RXRAT( 137 )      ! OPEN+O3=0.03*XO2+0.62*C2O3+...
     &                 +  0.500 * RXRAT( 138 )      ! OH+XYL=0.5*XO2+0.7*HO2+...
     &                 +          RXRAT( 139 )      ! OH+MGLY=XO2+C2O3
     &                 +  0.250 * RXRAT( 141 )      ! O+ISOP=0.25*XO2+0.5*FORM+...
     &                 +  0.991 * RXRAT( 142 )      ! OH+ISOP=0.991*XO2+...
     &                 +  0.200 * RXRAT( 143 )      ! O3+ISOP=0.2*XO2+0.6*FORM+...
     &                 +          RXRAT( 144 )      ! NO3+ISOP=XO2+0.8*NTR+...
     &                 +  0.713 * RXRAT( 145 )      ! OH+ISPD=0.713*XO2+...
     &                 +  0.064 * RXRAT( 146 )      ! O3+ISPD=0.064*XO2+...
     &                 +  0.075 * RXRAT( 147 )      ! NO3+ISPD=0.075*XO2+...
     &                 +  0.700 * RXRAT( 148 )      ! ISPD=0.7*XO2+0.067*ALD2+...
     &                 +  1.250 * RXRAT( 150 )      ! TERP+OH=1.25*XO2+0.75*HO2+...
     &                 +  0.760 * RXRAT( 151 )      ! TERP+O3=0.76*XO2+0.07*HO2+...
     &                 +  1.030 * RXRAT( 152 )      ! TERP+NO3=1.03*XO2+0.28*HO2+...
     &                 +  0.100 * RXRAT( 154 )      ! OH+ETOH=0.1*XO2+0.9*ALD2+...
     &                 +  0.991 * RXRAT( 155 )      ! OH+ETHA=0.991*XO2+...
     &                 +          RXRAT( 156 )      ! NO2+ISOP=XO2+0.8*NTR+...
     &                 +  0.870 * RXRAT( 166 )      ! CL+PAR=0.87*XO2+HCL+...
     &                 +  0.991 * RXRAT( 167 )      ! CL+ETHA=0.991*XO2+...
     &                 +  2.000 * RXRAT( 168 )      ! CL+ETH=2*XO2+FMCL+HO2+FORM
     &                 +  2.000 * RXRAT( 169 )      ! CL+OLE=2*XO2+0.33*ALD2+...
     &                 +  1.700 * RXRAT( 170 )      ! CL+IOLE=1.7*XO2+0.7*FMCL+...
     &                 +          RXRAT( 171 )      ! CL+ISOP=XO2+0.15*HCL+HO2+...
      LOSS( XO2     ) =           RXRAT(  54 )      ! XO2+NO=NO2
     &                 +          RXRAT(  56 )      ! XO2+HO2=ROOH
     &                 +  2.000 * RXRAT(  58 )      ! XO2+XO2=
     &                 +          RXRAT(  60 )      ! XO2+XO2N=
     &                 +          RXRAT(  93 )      ! XO2+C2O3=0.9*MEO2+0.1*AACD
     &                 +          RXRAT( 109 )      ! XO2+CXO3=0.9*ALD2+0.1*AACD

      PROD( XO2N    ) =   0.130 * RXRAT( 112 )      ! PAR+OH=0.13*XO2N+0.87*XO2+...
     &                 +  0.040 * RXRAT( 113 )      ! ROR=0.04*XO2N+0.6*ALD2+...
     &                 +  0.010 * RXRAT( 116 )      ! O+OLE=0.01*XO2N+0.3*ALDX+...
     &                 +  0.090 * RXRAT( 119 )      ! NO3+OLE=0.09*XO2N+FORM+...
     &                 +  0.088 * RXRAT( 142 )      ! OH+ISOP=0.088*XO2N+...
     &                 +  0.250 * RXRAT( 150 )      ! TERP+OH=0.25*XO2N+1.25*XO2+...
     &                 +  0.180 * RXRAT( 151 )      ! TERP+O3=0.18*XO2N+0.07*HO2+...
     &                 +  0.250 * RXRAT( 152 )      ! TERP+NO3=0.25*XO2N+...
     &                 +  0.009 * RXRAT( 155 )      ! OH+ETHA=0.009*XO2N+...
     &                 +  0.130 * RXRAT( 166 )      ! CL+PAR=0.13*XO2N+0.87*XO2+...
     &                 +  0.009 * RXRAT( 167 )      ! CL+ETHA=0.009*XO2N+...
      LOSS( XO2N    ) =           RXRAT(  55 )      ! XO2N+NO=NTR
     &                 +          RXRAT(  57 )      ! XO2N+HO2=ROOH
     &                 +  2.000 * RXRAT(  59 )      ! XO2N+XO2N=
     &                 +          RXRAT(  60 )      ! XO2N+XO2=

      PROD( NTR     ) =           RXRAT(  55 )      ! XO2N+NO=NTR
     &                 +          RXRAT( 115 )      ! ROR+NO2=NTR
     &                 +  0.100 * RXRAT( 129 )      ! TO2+NO=0.1*NTR+0.9*HO2+...
     &                 +          RXRAT( 133 )      ! CRO+NO2=NTR
     &                 +  0.800 * RXRAT( 144 )      ! NO3+ISOP=0.8*NTR+0.2*ISPD+...
     &                 +  0.850 * RXRAT( 147 )      ! NO3+ISPD=0.85*NTR+...
     &                 +  0.530 * RXRAT( 152 )      ! TERP+NO3=0.53*NTR+0.28*HO2+...
     &                 +  0.800 * RXRAT( 156 )      ! NO2+ISOP=0.8*NTR+0.2*ISPD+...
      LOSS( NTR     ) =           RXRAT(  61 )      ! NTR+OH=HNO3+HO2+0.33*FORM+...
     &                 +          RXRAT(  62 )      ! NTR=NO2+HO2+0.33*FORM+...

      PROD( ROOH    ) =           RXRAT(  56 )      ! XO2+HO2=ROOH
     &                 +          RXRAT(  57 )      ! XO2N+HO2=ROOH
      LOSS( ROOH    ) =           RXRAT(  63 )      ! ROOH+OH=XO2+0.5*ALD2+0.5*ALDX
     &                 +          RXRAT(  64 )      ! ROOH=OH+HO2+0.5*ALD2+0.5*ALDX

      PROD( FORM    ) =   0.330 * RXRAT(  61 )      ! NTR+OH=0.33*FORM+HO2+HNO3+...
     &                 +  0.330 * RXRAT(  62 )      ! NTR=0.33*FORM+HO2+NO2+...
     &                 +          RXRAT(  67 )      ! MEO2+NO=FORM+HO2+NO2
     &                 +  1.370 * RXRAT(  69 )      ! MEO2+MEO2=1.37*FORM+...
     &                 +          RXRAT(  71 )      ! MEPX=FORM+HO2+OH
     &                 +          RXRAT(  72 )      ! MEOH+OH=FORM+HO2
     &                 +          RXRAT(  79 )      ! HCO3=FORM+HO2
     &                 +          RXRAT(  92 )      ! C2O3+MEO2=FORM+0.9*HO2+...
     &                 +  0.100 * RXRAT( 108 )      ! CXO3+MEO2=0.1*FORM+0.9*XO2+...
     &                 +  0.200 * RXRAT( 116 )      ! O+OLE=0.2*FORM+0.3*ALDX+...
     &                 +  0.800 * RXRAT( 117 )      ! OH+OLE=0.8*FORM+0.33*ALD2+...
     &                 +  0.740 * RXRAT( 118 )      ! O3+OLE=0.74*FORM+0.18*ALD2+...
     &                 +          RXRAT( 119 )      ! NO3+OLE=FORM+NO2+0.91*XO2+...
     &                 +          RXRAT( 120 )      ! O+ETH=FORM+1.7*HO2+CO+...
     &                 +  1.560 * RXRAT( 121 )      ! OH+ETH=1.56*FORM+XO2+...
     &                 +          RXRAT( 122 )      ! O3+ETH=FORM+0.63*CO+...
     &                 +  2.000 * RXRAT( 123 )      ! NO3+ETH=2*FORM+XO2+NO2
     &                 +  0.250 * RXRAT( 126 )      ! IOLE+O3=0.25*FORM+...
     &                 +          RXRAT( 136 )      ! OPEN+OH=FORM+2*CO+2*HO2+...
     &                 +  0.700 * RXRAT( 137 )      ! OPEN+O3=0.7*FORM+0.62*C2O3+...
     &                 +  0.500 * RXRAT( 141 )      ! O+ISOP=0.5*FORM+0.75*ISPD+...
     &                 +  0.629 * RXRAT( 142 )      ! OH+ISOP=0.629*FORM+...
     &                 +  0.600 * RXRAT( 143 )      ! O3+ISOP=0.6*FORM+0.65*ISPD+...
     &                 +  0.167 * RXRAT( 145 )      ! OH+ISPD=0.167*FORM+...
     &                 +  0.150 * RXRAT( 146 )      ! O3+ISPD=0.15*FORM+...
     &                 +  0.282 * RXRAT( 147 )      ! NO3+ISPD=0.282*FORM+...
     &                 +  0.900 * RXRAT( 148 )      ! ISPD=0.9*FORM+0.067*ALD2+...
     &                 +  0.280 * RXRAT( 150 )      ! TERP+OH=0.28*FORM+1.25*XO2+...
     &                 +  0.240 * RXRAT( 151 )      ! TERP+O3=0.24*FORM+0.07*HO2+...
     &                 +  0.100 * RXRAT( 154 )      ! OH+ETOH=0.1*FORM+0.9*ALD2+...
     &                 +          RXRAT( 168 )      ! CL+ETH=FORM+2*XO2+HO2+FMCL
     &                 +          RXRAT( 175 )      ! CL+MEOH=FORM+HO2+HCL
      LOSS( FORM    ) =           RXRAT(  73 )      ! FORM+OH=HO2+CO
     &                 +          RXRAT(  74 )      ! FORM=2*HO2+CO
     &                 +          RXRAT(  75 )      ! FORM=CO
     &                 +          RXRAT(  76 )      ! FORM+O=OH+HO2+CO
     &                 +          RXRAT(  77 )      ! FORM+NO3=HNO3+HO2+CO
     &                 +          RXRAT(  78 )      ! FORM+HO2=HCO3
     &                 +          RXRAT( 172 )      ! FORM+CL=HCL+HO2+CO

      PROD( ALD2    ) =   0.330 * RXRAT(  61 )      ! NTR+OH=0.33*ALD2+HO2+...
     &                 +  0.330 * RXRAT(  62 )      ! NTR=0.33*ALD2+HO2+...
     &                 +  0.500 * RXRAT(  63 )      ! ROOH+OH=0.5*ALD2+XO2+0.5*ALDX
     &                 +  0.500 * RXRAT(  64 )      ! ROOH=0.5*ALD2+HO2+OH+0.5*ALDX
     &                 +          RXRAT( 102 )      ! CXO3+NO=ALD2+NO2+HO2+XO2
     &                 +          RXRAT( 106 )      ! PANX+OH=ALD2+NO2
     &                 +  0.900 * RXRAT( 108 )      ! CXO3+MEO2=0.9*ALD2+0.9*XO2+...
     &                 +  0.900 * RXRAT( 109 )      ! CXO3+XO2=0.9*ALD2+0.1*AACD
     &                 +  2.000 * RXRAT( 110 )      ! CXO3+CXO3=2*ALD2+2*XO2+2*HO2
     &                 +          RXRAT( 111 )      ! CXO3+C2O3=ALD2+XO2+HO2+MEO2
     &                 +  0.060 * RXRAT( 112 )      ! PAR+OH=0.06*ALD2+0.13*XO2N+...
     &                 +  0.600 * RXRAT( 113 )      ! ROR=0.6*ALD2+0.96*XO2+...
     &                 +  0.200 * RXRAT( 116 )      ! O+OLE=0.2*ALD2+0.3*ALDX+...
     &                 +  0.330 * RXRAT( 117 )      ! OH+OLE=0.33*ALD2+0.8*FORM+...
     &                 +  0.180 * RXRAT( 118 )      ! O3+OLE=0.18*ALD2+0.74*FORM+...
     &                 +  0.350 * RXRAT( 119 )      ! NO3+OLE=0.35*ALD2+FORM+...
     &                 +  1.240 * RXRAT( 124 )      ! IOLE+O=1.24*ALD2+0.66*ALDX+...
     &                 +  1.300 * RXRAT( 125 )      ! IOLE+OH=1.3*ALD2+0.7*ALDX+...
     &                 +  0.650 * RXRAT( 126 )      ! IOLE+O3=0.65*ALD2+...
     &                 +  1.180 * RXRAT( 127 )      ! IOLE+NO3=1.18*ALD2+...
     &                 +  0.252 * RXRAT( 145 )      ! OH+ISPD=0.252*ALD2+...
     &                 +  0.020 * RXRAT( 146 )      ! O3+ISPD=0.02*ALD2+...
     &                 +  0.067 * RXRAT( 148 )      ! ISPD=0.067*ALD2+0.333*CO+...
     &                 +  0.900 * RXRAT( 154 )      ! OH+ETOH=0.9*ALD2+HO2+...
     &                 +  0.991 * RXRAT( 155 )      ! OH+ETHA=0.991*ALD2+...
     &                 +  0.060 * RXRAT( 166 )      ! CL+PAR=0.06*ALD2+0.87*XO2+...
     &                 +  0.991 * RXRAT( 167 )      ! CL+ETHA=0.991*ALD2+HCL+...
     &                 +  0.330 * RXRAT( 169 )      ! CL+OLE=0.33*ALD2+FMCL+...
     &                 +  0.450 * RXRAT( 170 )      ! CL+IOLE=0.45*ALD2+0.7*FMCL+...
     &                 +          RXRAT( 176 )      ! CL+ETOH=ALD2+HO2+HCL
      LOSS( ALD2    ) =           RXRAT(  83 )      ! ALD2+O=C2O3+OH
     &                 +          RXRAT(  84 )      ! ALD2+OH=C2O3
     &                 +          RXRAT(  85 )      ! ALD2+NO3=C2O3+HNO3
     &                 +          RXRAT(  86 )      ! ALD2=MEO2+CO+HO2
     &                 +          RXRAT( 173 )      ! ALD2+CL=HCL+C2O3

      PROD( ALDX    ) =   0.330 * RXRAT(  61 )      ! NTR+OH=0.33*ALDX+HO2+...
     &                 +  0.330 * RXRAT(  62 )      ! NTR=0.33*ALDX+HO2+...
     &                 +  0.500 * RXRAT(  63 )      ! ROOH+OH=0.5*ALDX+0.5*ALD2+XO2
     &                 +  0.500 * RXRAT(  64 )      ! ROOH=0.5*ALDX+HO2+0.5*ALD2+OH
     &                 +  0.050 * RXRAT( 112 )      ! PAR+OH=0.05*ALDX+0.13*XO2N+...
     &                 +  0.500 * RXRAT( 113 )      ! ROR=0.5*ALDX+0.6*ALD2+...
     &                 +  0.300 * RXRAT( 116 )      ! O+OLE=0.3*ALDX+0.2*ALD2+...
     &                 +  0.620 * RXRAT( 117 )      ! OH+OLE=0.62*ALDX+0.33*ALD2+...
     &                 +  0.320 * RXRAT( 118 )      ! O3+OLE=0.32*ALDX+0.74*FORM+...
     &                 +  0.560 * RXRAT( 119 )      ! NO3+OLE=0.56*ALDX+FORM+...
     &                 +  0.220 * RXRAT( 121 )      ! OH+ETH=0.22*ALDX+1.56*FORM+...
     &                 +  0.660 * RXRAT( 124 )      ! IOLE+O=0.66*ALDX+1.24*ALD2+...
     &                 +  0.700 * RXRAT( 125 )      ! IOLE+OH=0.7*ALDX+1.3*ALD2+...
     &                 +  0.350 * RXRAT( 126 )      ! IOLE+O3=0.35*ALDX+...
     &                 +  0.640 * RXRAT( 127 )      ! IOLE+NO3=0.64*ALDX+...
     &                 +  0.030 * RXRAT( 137 )      ! OPEN+O3=0.03*ALDX+...
     &                 +  0.150 * RXRAT( 143 )      ! O3+ISOP=0.15*ALDX+0.6*FORM+...
     &                 +  0.800 * RXRAT( 144 )      ! NO3+ISOP=0.8*ALDX+0.8*NTR+...
     &                 +  0.120 * RXRAT( 145 )      ! OH+ISPD=0.12*ALDX+...
     &                 +  0.357 * RXRAT( 147 )      ! NO3+ISPD=0.357*ALDX+...
     &                 +  0.150 * RXRAT( 149 )      ! TERP+O=0.15*ALDX+5.12*PAR+...
     &                 +  0.470 * RXRAT( 150 )      ! TERP+OH=0.47*ALDX+1.25*XO2+...
     &                 +  0.210 * RXRAT( 151 )      ! TERP+O3=0.21*ALDX+0.07*HO2+...
     &                 +  0.470 * RXRAT( 152 )      ! TERP+NO3=0.47*ALDX+...
     &                 +  0.050 * RXRAT( 154 )      ! OH+ETOH=0.05*ALDX+0.9*ALD2+...
     &                 +  0.800 * RXRAT( 156 )      ! NO2+ISOP=0.8*ALDX+0.8*NTR+...
     &                 +  0.050 * RXRAT( 166 )      ! CL+PAR=0.05*ALDX+0.87*XO2+...
     &                 +  0.670 * RXRAT( 169 )      ! CL+OLE=0.67*ALDX+0.33*ALD2+...
     &                 +  0.550 * RXRAT( 170 )      ! CL+IOLE=0.55*ALDX+0.7*FMCL+...
      LOSS( ALDX    ) =           RXRAT(  98 )      ! ALDX+O=CXO3+OH
     &                 +          RXRAT(  99 )      ! ALDX+OH=CXO3
     &                 +          RXRAT( 100 )      ! ALDX+NO3=CXO3+HNO3
     &                 +          RXRAT( 101 )      ! ALDX=MEO2+CO+HO2
     &                 +          RXRAT( 174 )      ! ALDX+CL=HCL+CXO3

      PROD( PAR     ) =   0.200 * RXRAT( 116 )      ! O+OLE=0.2*PAR+0.3*ALDX+...
     &                 +  0.100 * RXRAT( 124 )      ! IOLE+O=0.1*PAR+0.66*ALDX+...
     &                 +  1.100 * RXRAT( 138 )      ! OH+XYL=1.1*PAR+0.5*XO2+...
     &                 +  0.250 * RXRAT( 141 )      ! O+ISOP=0.25*PAR+0.5*FORM+...
     &                 +  0.350 * RXRAT( 143 )      ! O3+ISOP=0.35*PAR+0.6*FORM+...
     &                 +  2.400 * RXRAT( 144 )      ! NO3+ISOP=2.4*PAR+0.8*NTR+...
     &                 +  1.565 * RXRAT( 145 )      ! OH+ISPD=1.565*PAR+...
     &                 +  0.360 * RXRAT( 146 )      ! O3+ISPD=0.36*PAR+0.15*FORM+...
     &                 +  1.282 * RXRAT( 147 )      ! NO3+ISPD=1.282*PAR+...
     &                 +  0.832 * RXRAT( 148 )      ! ISPD=0.832*PAR+0.067*ALD2+...
     &                 +  5.120 * RXRAT( 149 )      ! TERP+O=5.12*PAR+0.15*ALDX+...
     &                 +  1.660 * RXRAT( 150 )      ! TERP+OH=1.66*PAR+1.25*XO2+...
     &                 +  7.000 * RXRAT( 151 )      ! TERP+O3=7*PAR+0.07*HO2+...
     &                 +  2.400 * RXRAT( 156 )      ! NO2+ISOP=2.4*PAR+0.8*NTR+...
     &                 +  0.300 * RXRAT( 170 )      ! CL+IOLE=0.3*PAR+0.7*FMCL+...
      LOSS( PAR     ) =   0.660 * RXRAT(  61 )      ! NTR+OH=-0.66*PAR+HO2+...
     &                 +  0.660 * RXRAT(  62 )      ! NTR=-0.66*PAR+HO2+...
     &                 +  1.110 * RXRAT( 112 )      ! PAR+OH=-0.11*PAR+0.13*XO2N+...
     &                 +  2.100 * RXRAT( 113 )      ! ROR=-2.1*PAR+0.6*ALD2+...
     &                 +  0.700 * RXRAT( 117 )      ! OH+OLE=-0.7*PAR+0.33*ALD2+...
     &                 +          RXRAT( 118 )      ! O3+OLE=-PAR+0.74*FORM+...
     &                 +          RXRAT( 119 )      ! NO3+OLE=-PAR+FORM+0.91*XO2+...
     &                 +  1.110 * RXRAT( 166 )      ! PAR+CL=-0.11*PAR+0.87*XO2+...
     &                 +          RXRAT( 169 )      ! CL+OLE=-PAR+0.33*ALD2+...

      PROD( CO      ) =           RXRAT(  73 )      ! FORM+OH=CO+HO2
     &                 +          RXRAT(  74 )      ! FORM=CO+2*HO2
     &                 +          RXRAT(  75 )      ! FORM=CO
     &                 +          RXRAT(  76 )      ! FORM+O=CO+HO2+OH
     &                 +          RXRAT(  77 )      ! FORM+NO3=CO+HO2+HNO3
     &                 +          RXRAT(  86 )      ! ALD2=CO+MEO2+HO2
     &                 +          RXRAT( 101 )      ! ALDX=CO+MEO2+HO2
     &                 +  0.200 * RXRAT( 116 )      ! O+OLE=0.2*CO+0.3*ALDX+...
     &                 +  0.330 * RXRAT( 118 )      ! O3+OLE=0.33*CO+0.74*FORM+...
     &                 +          RXRAT( 120 )      ! O+ETH=CO+1.7*HO2+FORM+...
     &                 +  0.630 * RXRAT( 122 )      ! O3+ETH=0.63*CO+FORM+...
     &                 +  0.100 * RXRAT( 124 )      ! IOLE+O=0.1*CO+0.66*ALDX+...
     &                 +  0.250 * RXRAT( 126 )      ! IOLE+O3=0.25*CO+0.35*ALDX+...
     &                 +          RXRAT( 135 )      ! OPEN=CO+HO2+C2O3
     &                 +  2.000 * RXRAT( 136 )      ! OPEN+OH=2*CO+XO2+2*HO2+...
     &                 +  0.690 * RXRAT( 137 )      ! OPEN+O3=0.69*CO+0.62*C2O3+...
     &                 +          RXRAT( 140 )      ! MGLY=CO+HO2+C2O3
     &                 +  0.066 * RXRAT( 143 )      ! O3+ISOP=0.066*CO+0.6*FORM+...
     &                 +  0.334 * RXRAT( 145 )      ! OH+ISPD=0.334*CO+...
     &                 +  0.225 * RXRAT( 146 )      ! O3+ISPD=0.225*CO+0.15*FORM+...
     &                 +  0.643 * RXRAT( 147 )      ! NO3+ISPD=0.643*CO+...
     &                 +  0.333 * RXRAT( 148 )      ! ISPD=0.333*CO+0.067*ALD2+...
     &                 +  0.001 * RXRAT( 151 )      ! TERP+O3=0.001*CO+0.07*HO2+...
     &                 +          RXRAT( 163 )      ! OH+FMCL=CO+CL
     &                 +          RXRAT( 164 )      ! FMCL=CO+CL+HO2
     &                 +          RXRAT( 172 )      ! CL+FORM=CO+HO2+HCL
      LOSS( CO      ) =           RXRAT(  65 )      ! CO+OH=HO2

      PROD( MEO2    ) =           RXRAT(  66 )      ! OH=MEO2
     &                 +  0.700 * RXRAT(  70 )      ! MEPX+OH=0.7*MEO2+0.3*XO2+...
     &                 +          RXRAT(  86 )      ! ALD2=MEO2+CO+HO2
     &                 +          RXRAT(  87 )      ! C2O3+NO=MEO2+NO2
     &                 +  0.900 * RXRAT(  93 )      ! C2O3+XO2=0.9*MEO2+0.1*AACD
     &                 +  2.000 * RXRAT(  94 )      ! C2O3+C2O3=2*MEO2
     &                 +          RXRAT(  96 )      ! PACD=MEO2+OH
     &                 +          RXRAT(  97 )      ! AACD+OH=MEO2
     &                 +          RXRAT( 101 )      ! ALDX=MEO2+CO+HO2
     &                 +          RXRAT( 111 )      ! CXO3+C2O3=MEO2+XO2+HO2+ALD2
     &                 +          RXRAT( 165 )      ! CL=MEO2+HCL
      LOSS( MEO2    ) =           RXRAT(  67 )      ! MEO2+NO=FORM+HO2+NO2
     &                 +          RXRAT(  68 )      ! MEO2+HO2=MEPX
     &                 +  2.000 * RXRAT(  69 )      ! MEO2+MEO2=1.37*FORM+...
     &                 +  0.100 * RXRAT(  92 )      ! MEO2+C2O3=0.9*MEO2+0.9*HO2+...
     &                 +          RXRAT( 108 )      ! MEO2+CXO3=0.9*ALD2+0.9*XO2+...

      PROD( MEPX    ) =           RXRAT(  68 )      ! MEO2+HO2=MEPX
     &                 +          RXRAT(  81 )      ! HCO3+HO2=MEPX
      LOSS( MEPX    ) =           RXRAT(  70 )      ! MEPX+OH=0.7*MEO2+0.3*XO2+...
     &                 +          RXRAT(  71 )      ! MEPX=FORM+HO2+OH

      PROD( MEOH    ) =   0.630 * RXRAT(  69 )      ! MEO2+MEO2=0.63*MEOH+...
      LOSS( MEOH    ) =           RXRAT(  72 )      ! MEOH+OH=FORM+HO2
     &                 +          RXRAT( 175 )      ! MEOH+CL=HCL+HO2+FORM

      PROD( HCO3    ) =           RXRAT(  78 )      ! FORM+HO2=HCO3
      LOSS( HCO3    ) =           RXRAT(  79 )      ! HCO3=FORM+HO2
     &                 +          RXRAT(  80 )      ! HCO3+NO=FACD+NO2+HO2
     &                 +          RXRAT(  81 )      ! HCO3+HO2=MEPX

      PROD( FACD    ) =           RXRAT(  80 )      ! HCO3+NO=FACD+NO2+HO2
     &                 +  0.370 * RXRAT( 122 )      ! O3+ETH=0.37*FACD+0.63*CO+...
      LOSS( FACD    ) =           RXRAT(  82 )      ! FACD+OH=HO2

      PROD( PACD    ) =   0.800 * RXRAT(  91 )      ! C2O3+HO2=0.8*PACD+0.2*AACD+...
     &                 +  0.800 * RXRAT( 107 )      ! CXO3+HO2=0.8*PACD+0.2*AACD+...
      LOSS( PACD    ) =           RXRAT(  95 )      ! PACD+OH=C2O3
     &                 +          RXRAT(  96 )      ! PACD=MEO2+OH

      PROD( AACD    ) =   0.200 * RXRAT(  91 )      ! C2O3+HO2=0.2*AACD+0.8*PACD+...
     &                 +  0.100 * RXRAT(  92 )      ! C2O3+MEO2=0.1*AACD+0.9*HO2+...
     &                 +  0.100 * RXRAT(  93 )      ! C2O3+XO2=0.1*AACD+0.9*MEO2
     &                 +  0.200 * RXRAT( 107 )      ! CXO3+HO2=0.2*AACD+0.8*PACD+...
     &                 +  0.100 * RXRAT( 108 )      ! CXO3+MEO2=0.1*AACD+0.9*XO2+...
     &                 +  0.100 * RXRAT( 109 )      ! CXO3+XO2=0.1*AACD+0.9*ALD2
      LOSS( AACD    ) =           RXRAT(  97 )      ! AACD+OH=MEO2

      PROD( CXO3    ) =           RXRAT(  98 )      ! ALDX+O=CXO3+OH
     &                 +          RXRAT(  99 )      ! ALDX+OH=CXO3
     &                 +          RXRAT( 100 )      ! ALDX+NO3=CXO3+HNO3
     &                 +          RXRAT( 104 )      ! PANX=CXO3+NO2
     &                 +          RXRAT( 105 )      ! PANX=CXO3+NO2
     &                 +  0.250 * RXRAT( 141 )      ! O+ISOP=0.25*CXO3+0.5*FORM+...
     &                 +  0.200 * RXRAT( 143 )      ! O3+ISOP=0.2*CXO3+0.6*FORM+...
     &                 +  0.250 * RXRAT( 145 )      ! OH+ISPD=0.25*CXO3+...
     &                 +  0.075 * RXRAT( 147 )      ! NO3+ISPD=0.075*CXO3+...
     &                 +  0.390 * RXRAT( 151 )      ! TERP+O3=0.39*CXO3+0.07*HO2+...
     &                 +          RXRAT( 174 )      ! CL+ALDX=CXO3+HCL
      LOSS( CXO3    ) =           RXRAT( 102 )      ! CXO3+NO=ALD2+NO2+HO2+XO2
     &                 +          RXRAT( 103 )      ! CXO3+NO2=PANX
     &                 +          RXRAT( 107 )      ! CXO3+HO2=0.8*PACD+0.2*AACD+...
     &                 +          RXRAT( 108 )      ! CXO3+MEO2=0.9*ALD2+0.9*XO2+...
     &                 +          RXRAT( 109 )      ! CXO3+XO2=0.9*ALD2+0.1*AACD
     &                 +  2.000 * RXRAT( 110 )      ! CXO3+CXO3=2*ALD2+2*XO2+2*HO2
     &                 +          RXRAT( 111 )      ! CXO3+C2O3=MEO2+XO2+HO2+ALD2

      PROD( PANX    ) =           RXRAT( 103 )      ! CXO3+NO2=PANX
      LOSS( PANX    ) =           RXRAT( 104 )      ! PANX=CXO3+NO2
     &                 +          RXRAT( 105 )      ! PANX=CXO3+NO2
     &                 +          RXRAT( 106 )      ! PANX+OH=ALD2+NO2

      PROD( ROR     ) =   0.760 * RXRAT( 112 )      ! PAR+OH=0.76*ROR+0.13*XO2N+...
     &                 +  0.760 * RXRAT( 166 )      ! CL+PAR=0.76*ROR+0.87*XO2+...
      LOSS( ROR     ) =   0.980 * RXRAT( 113 )      ! ROR=0.02*ROR+0.6*ALD2+...
     &                 +          RXRAT( 114 )      ! ROR=HO2
     &                 +          RXRAT( 115 )      ! ROR+NO2=NTR

      PROD( OLE     ) =   0.300 * RXRAT( 170 )      ! CL+IOLE=0.3*OLE+0.7*FMCL+...
      LOSS( OLE     ) =           RXRAT( 116 )      ! OLE+O=0.2*ALD2+0.3*ALDX+...
     &                 +          RXRAT( 117 )      ! OLE+OH=0.8*FORM+0.33*ALD2+...
     &                 +          RXRAT( 118 )      ! OLE+O3=0.18*ALD2+0.74*FORM+...
     &                 +          RXRAT( 119 )      ! OLE+NO3=NO2+FORM+0.91*XO2+...
     &                 +          RXRAT( 169 )      ! OLE+CL=FMCL+0.33*ALD2+...

      PROD( ETH     ) =   0.0
      LOSS( ETH     ) =           RXRAT( 120 )      ! ETH+O=FORM+1.7*HO2+CO+...
     &                 +          RXRAT( 121 )      ! ETH+OH=XO2+1.56*FORM+...
     &                 +          RXRAT( 122 )      ! ETH+O3=FORM+0.63*CO+...
     &                 +          RXRAT( 123 )      ! ETH+NO3=NO2+XO2+2*FORM
     &                 +          RXRAT( 168 )      ! ETH+CL=FMCL+2*XO2+HO2+FORM

      PROD( IOLE    ) =   0.0
      LOSS( IOLE    ) =           RXRAT( 124 )      ! IOLE+O=1.24*ALD2+0.66*ALDX+...
     &                 +          RXRAT( 125 )      ! IOLE+OH=1.3*ALD2+0.7*ALDX+...
     &                 +          RXRAT( 126 )      ! IOLE+O3=0.65*ALD2+...
     &                 +          RXRAT( 127 )      ! IOLE+NO3=1.18*ALD2+...
     &                 +          RXRAT( 170 )      ! IOLE+CL=0.3*HCL+0.7*FMCL+...

      PROD( TOL     ) =   0.0
      LOSS( TOL     ) =           RXRAT( 128 )      ! TOL+OH=0.44*HO2+0.08*XO2+...

      PROD( CRES    ) =   0.360 * RXRAT( 128 )      ! TOL+OH=0.36*CRES+0.08*XO2+...
     &                 +          RXRAT( 130 )      ! TO2=CRES+HO2
     &                 +          RXRAT( 134 )      ! CRO+HO2=CRES
     &                 +  0.200 * RXRAT( 138 )      ! OH+XYL=0.2*CRES+0.5*XO2+...
      LOSS( CRES    ) =           RXRAT( 131 )      ! CRES+OH=0.4*CRO+0.6*XO2+...
     &                 +          RXRAT( 132 )      ! CRES+NO3=CRO+HNO3

      PROD( TO2     ) =   0.560 * RXRAT( 128 )      ! TOL+OH=0.56*TO2+0.08*XO2+...
     &                 +  0.300 * RXRAT( 138 )      ! OH+XYL=0.3*TO2+0.5*XO2+...
      LOSS( TO2     ) =           RXRAT( 129 )      ! TO2+NO=0.9*NO2+0.9*HO2+...
     &                 +          RXRAT( 130 )      ! TO2=CRES+HO2

      PROD( TOLRO2  ) =   0.765 * RXRAT( 128 )      ! TOL+OH=0.765*TOLRO2+...
      LOSS( TOLRO2  ) =           RXRAT( 178 )      ! TOLRO2+NO=NO+TOLNRXN
     &                 +          RXRAT( 179 )      ! TOLRO2+HO2=HO2+TOLHRXN

      PROD( OPEN    ) =   0.900 * RXRAT( 129 )      ! TO2+NO=0.9*OPEN+0.9*HO2+...
     &                 +  0.300 * RXRAT( 131 )      ! OH+CRES=0.3*OPEN+0.6*XO2+...
      LOSS( OPEN    ) =           RXRAT( 135 )      ! OPEN=C2O3+HO2+CO
     &                 +          RXRAT( 136 )      ! OPEN+OH=XO2+2*CO+2*HO2+...
     &                 +          RXRAT( 137 )      ! OPEN+O3=0.03*ALDX+...

      PROD( CRO     ) =   0.400 * RXRAT( 131 )      ! OH+CRES=0.4*CRO+0.6*XO2+...
     &                 +          RXRAT( 132 )      ! CRES+NO3=CRO+HNO3
      LOSS( CRO     ) =           RXRAT( 133 )      ! CRO+NO2=NTR
     &                 +          RXRAT( 134 )      ! CRO+HO2=CRES

      PROD( MGLY    ) =   0.200 * RXRAT( 137 )      ! OPEN+O3=0.2*MGLY+0.62*C2O3+...
     &                 +  0.800 * RXRAT( 138 )      ! OH+XYL=0.8*MGLY+0.5*XO2+...
     &                 +  0.168 * RXRAT( 145 )      ! OH+ISPD=0.168*MGLY+...
     &                 +  0.850 * RXRAT( 146 )      ! O3+ISPD=0.85*MGLY+...
      LOSS( MGLY    ) =           RXRAT( 139 )      ! MGLY+OH=XO2+C2O3
     &                 +          RXRAT( 140 )      ! MGLY=C2O3+HO2+CO

      PROD( XYL     ) =   0.0
      LOSS( XYL     ) =           RXRAT( 138 )      ! XYL+OH=0.7*HO2+0.5*XO2+...

      PROD( XYLRO2  ) =   0.804 * RXRAT( 138 )      ! OH+XYL=0.804*XYLRO2+...
      LOSS( XYLRO2  ) =           RXRAT( 180 )      ! XYLRO2+NO=NO+XYLNRXN
     &                 +          RXRAT( 181 )      ! XYLRO2+HO2=HO2+XYLHRXN

      PROD( ISOP    ) =   0.0
      LOSS( ISOP    ) =           RXRAT( 141 )      ! ISOP+O=0.75*ISPD+0.5*FORM+...
     &                 +          RXRAT( 142 )      ! ISOP+OH=0.912*ISPD+...
     &                 +          RXRAT( 143 )      ! ISOP+O3=0.65*ISPD+0.6*FORM+...
     &                 +          RXRAT( 144 )      ! ISOP+NO3=0.2*ISPD+0.8*NTR+...
     &                 +          RXRAT( 156 )      ! ISOP+NO2=0.2*ISPD+0.8*NTR+...
     &                 +          RXRAT( 171 )      ! ISOP+CL=0.15*HCL+XO2+HO2+...

      PROD( ISPD    ) =   0.750 * RXRAT( 141 )      ! O+ISOP=0.75*ISPD+0.5*FORM+...
     &                 +  0.912 * RXRAT( 142 )      ! OH+ISOP=0.912*ISPD+...
     &                 +  0.650 * RXRAT( 143 )      ! O3+ISOP=0.65*ISPD+0.6*FORM+...
     &                 +  0.200 * RXRAT( 144 )      ! NO3+ISOP=0.2*ISPD+0.8*NTR+...
     &                 +  0.200 * RXRAT( 156 )      ! NO2+ISOP=0.2*ISPD+0.8*NTR+...
     &                 +          RXRAT( 171 )      ! CL+ISOP=ISPD+XO2+HO2+...
      LOSS( ISPD    ) =           RXRAT( 145 )      ! ISPD+OH=1.565*PAR+...
     &                 +          RXRAT( 146 )      ! ISPD+O3=0.114*C2O3+...
     &                 +          RXRAT( 147 )      ! ISPD+NO3=0.357*ALDX+...
     &                 +          RXRAT( 148 )      ! ISPD=0.333*CO+0.067*ALD2+...

      PROD( ISOPRXN ) =           RXRAT( 142 )      ! OH+ISOP=ISOPRXN+0.629*FORM+...
      LOSS( ISOPRXN ) =   0.0

      PROD( TERP    ) =   0.0
      LOSS( TERP    ) =           RXRAT( 149 )      ! TERP+O=0.15*ALDX+5.12*PAR+...
     &                 +          RXRAT( 150 )      ! TERP+OH=0.75*HO2+1.25*XO2+...
     &                 +          RXRAT( 151 )      ! TERP+O3=0.57*OH+0.07*HO2+...
     &                 +          RXRAT( 152 )      ! TERP+NO3=0.47*NO2+0.28*HO2+...

      PROD( TRPRXN  ) =           RXRAT( 149 )      ! TERP+O=TRPRXN+5.12*PAR+...
     &                 +          RXRAT( 150 )      ! TERP+OH=TRPRXN+1.25*XO2+...
     &                 +          RXRAT( 151 )      ! TERP+O3=TRPRXN+0.07*HO2+...
     &                 +          RXRAT( 152 )      ! TERP+NO3=TRPRXN+0.28*HO2+...
      LOSS( TRPRXN  ) =   0.0

      PROD( SO2     ) =   0.0
      LOSS( SO2     ) =           RXRAT( 153 )      ! SO2+OH=SULF+HO2+SULRXN

      PROD( SULF    ) =           RXRAT( 153 )      ! SO2+OH=SULF+HO2+SULRXN
      LOSS( SULF    ) =   0.0

      PROD( SULRXN  ) =           RXRAT( 153 )      ! SO2+OH=SULRXN+HO2+SULF
      LOSS( SULRXN  ) =   0.0

      PROD( ETOH    ) =   0.0
      LOSS( ETOH    ) =           RXRAT( 154 )      ! ETOH+OH=HO2+0.9*ALD2+...
     &                 +          RXRAT( 176 )      ! ETOH+CL=HCL+HO2+ALD2

      PROD( ETHA    ) =   0.0
      LOSS( ETHA    ) =           RXRAT( 155 )      ! ETHA+OH=0.991*ALD2+...
     &                 +          RXRAT( 167 )      ! ETHA+CL=HCL+0.991*ALD2+...

      PROD( CL2     ) =   0.300 * RXRAT( 160 )      ! CLO+CLO=0.3*CL2+1.4*CL
      LOSS( CL2     ) =           RXRAT( 157 )      ! CL2=2*CL

      PROD( CL      ) =   2.000 * RXRAT( 157 )      ! CL2=2*CL
     &                 +          RXRAT( 158 )      ! HOCL=CL+OH
     &                 +  1.400 * RXRAT( 160 )      ! CLO+CLO=1.4*CL+0.3*CL2
     &                 +          RXRAT( 161 )      ! CLO+NO=CL+NO2
     &                 +          RXRAT( 163 )      ! OH+FMCL=CL+CO
     &                 +          RXRAT( 164 )      ! FMCL=CL+CO+HO2
     &                 +          RXRAT( 177 )      ! HCL+OH=CL
      LOSS( CL      ) =           RXRAT( 159 )      ! CL+O3=CLO
     &                 +          RXRAT( 165 )      ! CL=HCL+MEO2
     &                 +          RXRAT( 166 )      ! CL+PAR=HCL+0.87*XO2+...
     &                 +          RXRAT( 167 )      ! CL+ETHA=HCL+0.991*ALD2+...
     &                 +          RXRAT( 168 )      ! CL+ETH=FMCL+2*XO2+HO2+FORM
     &                 +          RXRAT( 169 )      ! CL+OLE=FMCL+0.33*ALD2+...
     &                 +          RXRAT( 170 )      ! CL+IOLE=0.3*HCL+0.7*FMCL+...
     &                 +          RXRAT( 171 )      ! CL+ISOP=0.15*HCL+XO2+HO2+...
     &                 +          RXRAT( 172 )      ! CL+FORM=HCL+HO2+CO
     &                 +          RXRAT( 173 )      ! CL+ALD2=HCL+C2O3
     &                 +          RXRAT( 174 )      ! CL+ALDX=HCL+CXO3
     &                 +          RXRAT( 175 )      ! CL+MEOH=HCL+HO2+FORM
     &                 +          RXRAT( 176 )      ! CL+ETOH=HCL+HO2+ALD2

      PROD( HOCL    ) =           RXRAT( 162 )      ! CLO+HO2=HOCL
      LOSS( HOCL    ) =           RXRAT( 158 )      ! HOCL=OH+CL

      PROD( CLO     ) =           RXRAT( 159 )      ! CL+O3=CLO
      LOSS( CLO     ) =   2.000 * RXRAT( 160 )      ! CLO+CLO=0.3*CL2+1.4*CL
     &                 +          RXRAT( 161 )      ! CLO+NO=CL+NO2
     &                 +          RXRAT( 162 )      ! CLO+HO2=HOCL

      PROD( FMCL    ) =           RXRAT( 168 )      ! CL+ETH=FMCL+2*XO2+HO2+FORM
     &                 +          RXRAT( 169 )      ! CL+OLE=FMCL+0.33*ALD2+...
     &                 +  0.700 * RXRAT( 170 )      ! CL+IOLE=0.7*FMCL+0.3*HCL+...
     &                 +  0.850 * RXRAT( 171 )      ! CL+ISOP=0.85*FMCL+XO2+HO2+...
      LOSS( FMCL    ) =           RXRAT( 163 )      ! FMCL+OH=CL+CO
     &                 +          RXRAT( 164 )      ! FMCL=CL+CO+HO2

      PROD( HCL     ) =           RXRAT( 165 )      ! CL=HCL+MEO2
     &                 +          RXRAT( 166 )      ! CL+PAR=HCL+0.87*XO2+...
     &                 +          RXRAT( 167 )      ! CL+ETHA=HCL+0.991*ALD2+...
     &                 +  0.300 * RXRAT( 170 )      ! CL+IOLE=0.3*HCL+0.7*FMCL+...
     &                 +  0.150 * RXRAT( 171 )      ! CL+ISOP=0.15*HCL+XO2+HO2+...
     &                 +          RXRAT( 172 )      ! CL+FORM=HCL+HO2+CO
     &                 +          RXRAT( 173 )      ! CL+ALD2=HCL+C2O3
     &                 +          RXRAT( 174 )      ! CL+ALDX=HCL+CXO3
     &                 +          RXRAT( 175 )      ! CL+MEOH=HCL+HO2+FORM
     &                 +          RXRAT( 176 )      ! CL+ETOH=HCL+HO2+ALD2
      LOSS( HCL     ) =           RXRAT( 177 )      ! HCL+OH=CL

      PROD( TOLNRXN ) =           RXRAT( 178 )      ! TOLRO2+NO=TOLNRXN+NO
      LOSS( TOLNRXN ) =   0.0

      PROD( TOLHRXN ) =           RXRAT( 179 )      ! TOLRO2+HO2=TOLHRXN+HO2
      LOSS( TOLHRXN ) =   0.0

      PROD( XYLNRXN ) =           RXRAT( 180 )      ! XYLRO2+NO=XYLNRXN+NO
      LOSS( XYLNRXN ) =   0.0

      PROD( XYLHRXN ) =           RXRAT( 181 )      ! XYLRO2+HO2=XYLHRXN+HO2
      LOSS( XYLHRXN ) =   0.0

      PROD( BENZENE ) =   0.0
      LOSS( BENZENE ) =           RXRAT( 182 )      ! BENZENE+OH=OH+0.764*BENZRO2

      PROD( BENZRO2 ) =   0.764 * RXRAT( 182 )      ! BENZENE+OH=0.764*BENZRO2+OH
      LOSS( BENZRO2 ) =           RXRAT( 183 )      ! BENZRO2+NO=NO+BNZNRXN
     &                 +          RXRAT( 184 )      ! BENZRO2+HO2=HO2+BNZHRXN

      PROD( BNZNRXN ) =           RXRAT( 183 )      ! BENZRO2+NO=BNZNRXN+NO
      LOSS( BNZNRXN ) =   0.0

      PROD( BNZHRXN ) =           RXRAT( 184 )      ! BENZRO2+HO2=BNZHRXN+HO2
      LOSS( BNZHRXN ) =   0.0

      PROD( SESQ    ) =   0.0
      LOSS( SESQ    ) =           RXRAT( 185 )      ! SESQ+O3=O3+SESQRXN
     &                 +          RXRAT( 186 )      ! SESQ+OH=OH+SESQRXN
     &                 +          RXRAT( 187 )      ! SESQ+NO3=NO3+SESQRXN

      PROD( SESQRXN ) =           RXRAT( 185 )      ! SESQ+O3=SESQRXN+O3
     &                 +          RXRAT( 186 )      ! SESQ+OH=SESQRXN+OH
     &                 +          RXRAT( 187 )      ! SESQ+NO3=SESQRXN+NO3
      LOSS( SESQRXN ) =   0.0

      PNEG( PAR ) =   0.660 * RXRAT(  61 )      ! NTR+OH=-0.66*PAR+HO2+...
     &             +  0.660 * RXRAT(  62 )      ! NTR=-0.66*PAR+HO2+...
     &             +  2.100 * RXRAT( 113 )      ! ROR=-2.1*PAR+0.6*ALD2+...
     &             +  0.700 * RXRAT( 117 )      ! OH+OLE=-0.7*PAR+0.33*ALD2+...
     &             +          RXRAT( 118 )      ! O3+OLE=-PAR+0.74*FORM+...
     &             +          RXRAT( 119 )      ! NO3+OLE=-PAR+FORM+0.91*XO2+...
     &             +          RXRAT( 169 )      ! CL+OLE=-PAR+0.33*ALD2+...

      RETURN

      END
